import java.io.*;

/**
 * A RecordEnum that can be instantiated from a file.
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 1.1 1998-11-22
 */
public class FileRecordEnum extends RecordEnum{
	protected BufferedReader _input;
	protected String _fileName;
	
	public FileRecordEnum(String fileName)
		throws FileNotFoundException
	{
		this._input = new BufferedReader(new FileReader(fileName));
		this._fileName = fileName;
	}
	
	public boolean hasMoreElements() {
		try {
			return this._input.ready();
		} catch (IOException err) {
			return false;
		}
	}
	
	/**
	 * In this implementation we assume that records are
	 * separated by blank lines, so we accumulate lines
	 * from our source file until we encounter a line of length 0.
	 */
	public Object nextElement() {
		StringBuffer record = new StringBuffer();
	
		try {
			String line;
			if (this.hasMoreElements()) {
				do {
					line = this._input.readLine();
					// BufferedReader.readLine() strips off
					// the trailing newline, so we must put it back!
					record.append(line).append('\n');
				} while (this.hasMoreElements() && line.length() > 0);
			}
		} catch (IOException err) {
			// Enumeration doesn't expect us to raise an exception.
			this.IOerror();
		}
		return record.toString();
	}

	/**
	 * Shouldn't happen, but leave a hook for subclasses.
	 */
	public void IOerror() {
		System.err.println("IOException in FileRecordEnum " + this._fileName);
	}
}
