/**
 * Wraps a RecordEnum, filtering out records that do not
 * contain a given key.
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 1.0 1998-11-22
 */
public class FilterRecordEnum extends RecordEnum {
	
	protected RecordEnum _source;
	protected String _key;
	
	// To make hasMoreElements() and nextElement() as simple as possible,
	// we cache the next element that matches the key, if there is one.
	protected String _nextElement;
	
	FilterRecordEnum(String key, RecordEnum source) {
		this._source = source;
		this._key = key;
		this.establishInvariant();
	}
	
	/**
	 * Invariant:
	 * <ul>
	 * <li>Either _nextElement contains the next matching record
	 * <li> or _source is exhausted.
	 * </ul>
	 */
	protected void establishInvariant() {
		if (this._nextElement == null) {
			while (this._source.hasMoreElements()) {
				String record = (String) this._source.nextElement();
				if (record.indexOf(this._key) > 0) {
					this._nextElement = record;
					return;
				}
			}
		}
	}
	
	public boolean hasMoreElements() {
		return this._nextElement != null;
	}
	
	public Object nextElement() {
		Object result = this._nextElement;
		this._nextElement = null;
		this.establishInvariant();
		return result;
	}
}
