/*
	Procedural record searching example for P2.
*/

import java.io.*;

/**
* An example of how <i>not</i> to program with Java.
* This program uses exclusively static methods to solve a
* problem procedurally.  The result is not easily extensible.
*
* @author Oscar.Nierstrasz@acm.org
* @version 1.0 1998-11-22
*/
public class ProceduralRec {

	public static void main(String args[]) {
		if (args.length != 2) { usage(); }
		else { recSearch(args[0], args[1]); }
	}

	public static void usage() {
		fatal("Usage: java ProceduralRec <search key> <refer file>");
	}

	public static void fatal(String msg) {
		System.err.println(msg);
		System.exit(1);
	}

	/**
	 * Search a file of records, printing those that contain the key.
	 */
	public static void recSearch(String key, String fileName) {
 		try {
			BufferedReader input = new BufferedReader(new FileReader(fileName));
			String record;
			do {
				record = getNextRecord(input);
				if (record.indexOf(key) > 0) {
					System.out.print(record);
				}
			} while (record.length() > 0);
		} catch(FileNotFoundException err) {
			fatal("Can't open: " + fileName);
		}
	}

	/**
	 * Always returns a String.  A record aleays contains at least
	 * one character ('\n').  Returns an empty String if at EOF.
	 */
	public static String getNextRecord(BufferedReader input) {
		StringBuffer record = new StringBuffer();

		try {
			String line;
			if (input.ready()) {
				do {
					line = input.readLine();
					record.append(line).append('\n');
				} while (input.ready() && line.length() > 0);
			}
		} catch (IOException err) {
			fatal("Can't read input");
		}
		return record.toString();
	}
}

