
/**
 * Demonstrates a textbook algorithm using a stack:
 * walk through a string containing various kinds of
 * parentheses (like "if (a) { b[c++]; } else { e(f); }")
 * and report whether the parentheses are balanced.
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 1.1 1999-04-10 -- added declarativ eversion of parenMatch()
 * 
 * version 1.0 1998-11-25
 */
 public class ParenMatch {

	String line_;
	StackInterface stack_;
	
	/**
	 * The expression (line) to test and the stack to use
	 * are parameters so we can try different stack implementations.
	 * There should be a default constructor ...
	 */
	public ParenMatch(String line, StackInterface stack) {
		line_ = line; stack_ = stack;
	}
	
	/**
	 * This classic algorithm pushes a value on the stack
	 * every time a left parenthesis is enountered, and
	 * pops a value when a right parenthis is enountered.
	 * <p>
	 * If and only if all the parentheses match,
	 * and if the stack is empty at the end,
	 * then everything is ok.
	 */
	public boolean oldParenMatch() throws AssertionException {
		for (int i=0; i<line_.length(); i++) {
			char c = line_.charAt(i);
			switch (c) {
			case '{' : stack_.push(new Character('}')); break;
			case '(' : stack_.push(new Character(')')); break;
			case '[' : stack_.push(new Character(']')); break;
			case ']' : case ')' : case '}' :
				if (stack_.isEmpty()) { return false; }
				if (((Character) stack_.top()).charValue() == c) {
					stack_.pop();
				} else { return false; }
				break;
			default : break;
			}
		}
		return stack_.isEmpty();
	}

	/**
	 * This classic algorithm pushes a value on the stack
	 * every time a left parenthesis is enountered, and
	 * pops a value when a right parenthis is enountered.
	 * <p>
	 * If and only if all the parentheses match,
	 * and if the stack is empty at the end,
	 * then everything is ok.
	 * <P>
	 * Equivalent to oldParenMatch(), but rewritten in declarative style
	 */
	public boolean parenMatch() throws AssertionException {
		for (int i=0; i<line_.length(); i++) {
			char c = line_.charAt(i);
			if (isLeftParen(c)) {
				// expect matching right paren later
				stack_.push(new Character(matchingRightParen(c)));
			} else {
				if (isRightParen(c)) {
					// empty stack => missing left paren
					if (stack_.isEmpty()) { return false; }
					if (stack_.top().equals(new Character(c))) {
						stack_.pop();
					} else { return false; } // mismatched paren
				}
			}
		}
		return stack_.isEmpty(); // not empty => missing right paren
	}
	
	private boolean isLeftParen(char c) {
		return (c == '(') || (c == '[') || (c == '{');
	}
	
	private boolean isRightParen(char c) {
		return (c == ')') || (c == ']') || (c == '}');
	}
	
	private char matchingRightParen(char c) {
		switch (c) {
			case '(' : return ')';
			case '[' : return ']';
			case '{' : return '}';
		}
		return c; // should we throw an exception if we get here?
	}

	/**
	 * Returns a printable result.
	 */
	public String reportMatch() throws AssertionException {
		if (line_ == null) { return ""; }
		return "\"" + line_ + "\" is"
			+ (this.parenMatch() ? " " : " not ")
			+ "balanced";
	}

}