
import java.io.*;

/**
 * Test driver for various Stack implementations
 * @author Oscar.Nierstrasz@acm.org
 * @version 1.0 1998-11-25
 */
public class TestStack {

	static public void main(String args[]) {
/* */
		testStack(new LinkStack());
/*		testStack(new ArrayStack());
		testStack(new SimpleWrappedStack());
		testStack(new WrappedStack());
*/
/*
		benchmark(new LinkStack());
		benchmark(new ArrayStack());
		// benchmark(new SimpleWrappedStack());
		benchmark(new WrappedStack());
*/
		// parenTestLoop(new ArrayStack());
	}
	
	/**
	 * Unit test for implementations of StackInterface.
	 * Exercises all the Stack methods.
	 */
	static public void testStack(StackInterface stack) {
		try {
			System.out.print("Testing " + stack.getClass().getName() + " ... ");
			assert(stack.isEmpty());
			for (int i=1; i<=10; i++) {
				stack.push(new Integer(i));
			}
			assert(!stack.isEmpty());
			assert(stack.size() == 10);
			assert(((Integer) stack.top()).intValue() == 10);
			for (int i=10; i>1; i--) {
				stack.pop();
			}
			assert(!stack.isEmpty());
			assert(stack.size() == 1);
			assert(((Integer) stack.top()).intValue() == 1);
			stack.pop();
			assert(stack.isEmpty());

			// How do we check that an assertion fails when it should?
			boolean emptyPopCaught = false;
			try {
				// we expect pop() to raise an exception
				stack.pop();
			} catch(AssertionException err) {
				// we should get here!
				emptyPopCaught = true;
			}
			assert(emptyPopCaught); // should be true

			System.out.println("passed all tests!");
		} catch (Exception err) { // NB: any kind!
			err.printStackTrace();
		} 
	}
	
	/**
	 * Times how long it takes to do a large number of pushes/pops.
	 */
	static public void benchmark(StackInterface stack) {
		int iterations = 100000; // 100K
		Object item = new Integer(0);
		Timer timer = new Timer();
		long pushTime, popTime;
		
		// System.out.println("Timing " + stack.getClass().getName());
		try {
			timer.reset();
			for (int i=0; i<iterations; i++) {
				stack.push(item);
			}
			pushTime = timer.timeElapsed();
			// System.out.println(pushTime + " milliseconds for " + iterations + " pushes");
			
			timer.reset();
			for (int i=0; i<iterations; i++) {
				stack.pop();
			}
			popTime = timer.timeElapsed();
			// System.out.println(popTime + " milliseconds for " + iterations + " pops");

			String run = "Apple MRJ";
			// String run = "Metrowerks";
			// String run = "Metrowerks JIT";
			System.out.println(run + '\t' + stack.getClass().getName() + '\t'
				+ pushTime + '\t' + popTime);
		
		} catch (AssertionException err) {
			err.printStackTrace();
		}
	}
	
	/**
	 * Test driver for the ParenMatch class.
	 */
	public static void parenTestLoop(StackInterface stack) {
		BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
		String line;
		try {
			System.out.println("Please enter parenthesized expressions to test");
			System.out.println("(empty line to stop)");
			do {
				line = in.readLine();
				System.out.println(new ParenMatch(line, stack).reportMatch());
			} while(line != null && line.length() > 0);
			System.out.println("bye!");
		} catch (IOException err) {
		} catch (AssertionException err) {
			err.printStackTrace();
		}
	}
	
	static private void assert(boolean assertion)
		throws AssertionException
	{
		if (!assertion) {
			throw new AssertionException("Assertion failed in TestStack");
		}
	}
}
