package tictactoe;

import java.awt.event.*;
import java.applet.Applet;

/**
 * This class picks up mouse clicks and propogates them
 * to a Player of the BoardGame.
 * <P>
 * <B>To do:</B>
 * Implement better feedback.
 *
 * @see Place
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 2.0 1999-03-26
 */
public class PlaceListener extends MouseAdapter {
	private final Place place_;
	private final GameApplet applet_;
	
	public PlaceListener(Place place, GameApplet applet) {
		place_ = place;
		applet_ = applet;
	}

	/**
	 * When the mouse is clicked, we ask the current player to
	 * make the corresponding move.  Report when the game is over.
	 */
	public void mouseClicked(MouseEvent e) {
		BoardGame game = applet_.game();
		int col = place_.col();
		int row = place_.row();
		if (game.notOver()) {
			try {
				((AppletPlayer) game.currentPlayer()).move(col,row);
				applet_.showFeedBack(game.currentPlayer().mark() + " plays");
			} catch (AssertionException err) {
				applet_.showFeedBack("Invalid move ignored (" + col + "," + row + ")"
					+ game.currentPlayer().mark() + " plays");
			}
			if (!game.notOver()) {
				applet_.showFeedBack("Game over -- " + game.winner() + " wins!");
			}
		} else {
			applet_.showFeedBack("The game is over!");
		}
	}
}
