package tictactoe;

import java.io.*;
import java.util.*;

/**
 * Interface for TicTacToe and Gomoku.
 * This is a subset of the BoardGame interface,
 * needed by remote clients.
 *
 * @see AbstractBoardGame
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 2.1 1999-03-31
 */
public interface RemoteGame {
	/**
	 * The game is ready to be played.
	 * (There are two connected clients.)
	 */
	public boolean ready();
	/**
	 * A client connected to the game must join
	 * the game to know what its mark is (X or O).
	 */
	public char join();
	/**
	 * @return the number of columns of this game
	 */
	public int cols();
	/**
	 * @return the number of rows
	 */
	public int rows();
	/**
	 * Attempt to move to the given square.
	 * Called by a remote client.  The Boardgame
	 * will check if move.mark corresponds to the
	 * current Player, and then call move().
	 */
	public void remoteMove(Move move)
		throws AssertionException;
	/**
	 * @return the Player whose turn it is.
	 */
	public Player currentPlayer();
	/**
	 * @return the winner of the game (possibly nobody)
	 */
	public Player winner();
	/**
	 * @return whether the game is over
	 */
	public boolean notOver();
	/**
	 * Subclasses should implement this method of java.util.Observable.
	 */
	public void addObserver(Observer o);
}
