package tictactoe;

import java.io.*;

/**
 * Helper class to find winning scores for BoardGame.
 * A Runner runs across a BoardGame, starting at the
 * position last played, and searches for a winning score.
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 1.6 1999-02-04
 **/
public class Runner {

	BoardGame game_;
	// Home col and row:
	int homeCol_;
	int homeRow_;
	// Current col & row:
	int col_=0;
	int row_=0;

	/**
	 * @param game the associated BoardGame
	 * @param col the column of the last position played
	 * @param row the row of the last position played
	 **/
	public Runner(BoardGame game, int col, int row)
	{
		game_ = game;
		homeCol_ = col;
		homeRow_ = row;
	}

	/**
	 * A Runner runs in some direction (dcol,drow)
	 * as far as it can, as long as the pieces on the
	 * Board are the same as the home square.
	 * Then it runs in the opposite direction, and
	 * returns the total score.
	 **/
	public int run(int dcol, int drow)
		throws AssertionException
	{
		int score = 1;
		this.goHome();
		score += this.forwardRun(dcol, drow);
		this.goHome();
		dcol = -dcol;
		drow = -drow;
		score += this.forwardRun(dcol, drow);
		debug("Runner(" + homeCol_ + "," + homeRow_ + ")" +
			".run(" + -dcol + "," + -drow + ") = " + score + "\n");
		return score;
	}
	
	/**
	 * Return to the "home" position (the last one played).
	 **/
	private void goHome() {
		col_= homeCol_;
		row_ = homeRow_;
	}
	
	/**
	 * Recursively move forward as long as we are
	 * in a run.  Return the length of the run.
	 * @param dcol increment in column to move
	 * @param drow increment in row to move
	 **/
	private int forwardRun(int dcol, int drow)
		throws AssertionException
	{
		this.move(dcol, drow);
		if (this.samePlayer())
			return 1 + this.forwardRun(dcol, drow);
		else
			return 0;
	}
	
	/**
	 * Move in the direction given by the increments.
	 **/
	private void move(int dcol, int drow) {
		col_ = col_ + dcol;
		row_ = row_ + drow;
		debug("Runner.move(" + col_ + "," + row_ + ")\n");
	}

	/**
	 * The player at the current location is the
	 * same as that of our home square.
	 **/
	private boolean samePlayer()
		throws AssertionException
	{
		if (game_.inRange(col_,row_)) {
			Player home = game_.get(homeCol_, homeRow_);
			Player here = game_.get(col_, row_);
			return home == here;
		} else {
			// Oops -- we walked off the edge of the board!
			return false;
		}
	}
	
	// Introduced this to fugure out why interactive
	// 10*5 Gomoku disn't find a winner after 5 in a row.
	// (Problem was GameApplet.makeGrid nested loop was backwards.)
	private void debug(String msg) {
		// System.err.print(msg);
	}
}
