package tictactoe;

import java.io.*;

/**
 * Test Driver for TicTacToe.  Instantiates a number of games
 * and checks if the final state and winner is as expected.
 * @author Oscar.Nierstrasz@acm.org
 * @version 2.0 1999-03-26
 **/
public class TestDriver {
	// Test output is verbose or quiet?
	private static boolean verbose_ = false; // true;

	public static void main(String args[]) {
		doTicTacToeTests();
		doGomokuTests();
	}
	
	public static void doGomokuTests() {
		System.err.println("Started Gomoku Unit Tests");
		// X wins on diagonal
		testGame(new Gomoku(
					new StreamPlayer('X', "\naa\n" // nonsense input
									+ "f6\ng5\ne7\nd8\nc9\n"),
					new StreamPlayer('O', "b2\nh4\nc3\nd4\n")),
					"X", (19*19-9));
		// X wins on row
		testGame(new Gomoku(
					new StreamPlayer('X', "b1\nc1\nd1\ne1\n" + "f1\ng1\n"),
					new StreamPlayer('O', "b2\nc2\nd2\ne2\nf2\n" + "g2\n"),
					10, 5),
					"X", (10*5-9));
		System.err.println("Finished Gomoku Unit Tests");
	}
	
	public static void doTicTacToeTests() {
		System.err.println("Started TicTacToe Unit Tests");
		// X wins on diagonal
		testGame(new TicTacToe(
					new StreamPlayer('X', "a1\nb2\nc3\n"),
					new StreamPlayer('O', "b1\nc1\n")),
					"X", 4);

		// no one wins
		testGame(new TicTacToe(
					new StreamPlayer('X', "b2\na1\nb3\na2\nc1\n"),
					new StreamPlayer('O', "a3\nc3\nb1\nc2\n")),
					"nobody", 0);

		// O wins on reverse diagonal
		// tries invalid moves
		testGame(new TicTacToe(
					new StreamPlayer('X', "a1\nb1\n"
									+ "a1\nb1\nb2\nc1\nd0\nxxx\n\n" // invalid moves
									+ "a2\n"),
					new StreamPlayer('O', "b2\nc1\na3\n")),
					"O", 3);

		// X wins on centre column
		testGame(new TicTacToe(
					new StreamPlayer('X', "b2\nb1\nb3\n"),
					new StreamPlayer('O', "a1\n"
									+ "b1\nA0\n" // invalid moves
									+ "a3\n")),
					"X", 4);

		// O wins on top row
		testGame(new TicTacToe(
					new StreamPlayer('X', "b2\na1\nc1\n"),
					new StreamPlayer('O',"a3\nc3\nb3\n")),
					"O", 3);

		System.err.println("Finished TicTacToe Unit Tests");
	}

	/**
	 * Run a game with simulated input for X and Y,
	 * and check that the winner is as expected.
	 **/
	public static void testGame(
		BoardGame game, String winner, int squaresLeft)
	{
		try {
			game.test();
			
			GameDriver.playGame(game, verbose_);
			
			assert(game.winner().toString().equals(winner));
			assert(game.squaresLeft() == squaresLeft);
		} catch (AssertionException err) {
			System.err.println("TestDriver assertion failed in\n" + game);
			err.printStackTrace();
		}
	}

	/**
	 * NB: This version of assert() is static.
	 **/
	private static void assert(boolean assertion)
		throws AssertionException
	{
		if (!assertion)
			throw new AssertionException("assertion failed in TestDriver");
	}
}