import junit.framework.*;

public class MoneyTest extends TestCase {
	private Money f12CHF;
	private Money f14CHF;
	
	private Money f7USD;
	private Money f21USD;
	private MoneyBag fMB1;
	private MoneyBag fMB2;

	public MoneyTest(String name) {
		super(name);
	}

	protected void setUp() {
		f12CHF = new Money(12, "CHF");
		f14CHF = new Money(14, "CHF");
		f7USD =	new Money( 7, "USD");
		f21USD = new Money(21, "USD");
		fMB1 = new MoneyBag(f12CHF, f7USD);
		fMB2 = new MoneyBag(f14CHF, f21USD);
	}

	public void testEquals() {
		assert(!f12CHF.equals(null));
		assertEquals(f12CHF, f12CHF);
		assertEquals(f12CHF, new Money(12, "CHF"));
		assert(!f12CHF.equals(f14CHF));
	}

	public void testSimpleAdd() {
		IMoney expected = new Money(26, "CHF");
		IMoney result = f12CHF.add(f14CHF);
		assertEquals(expected, result);
	}

	public void testBagEquals() {
		assert(!fMB1.equals(null));
		assertEquals(fMB1, fMB1);
		assert(!fMB1.equals(f12CHF));
		assert(!f12CHF.equals(fMB1));
		assert(!fMB1.equals(fMB2));
	}

	public void testMixedSimpleAdd() {
		// [12 CHF] + [7 USD] == {[12 CHF][7 USD]}
		Money bag[] = { f12CHF, f7USD };
		MoneyBag expected = new MoneyBag(bag);
		assertEquals(expected, f12CHF.add(f7USD));
	}

	public static Test suite() {
		TestSuite suite = new TestSuite();
		suite.addTest(new MoneyTest("testEquals"));
		suite.addTest(new MoneyTest("testSimpleAdd"));
		suite.addTest(new MoneyTest("testBagEquals"));
		suite.addTest(new MoneyTest("testMixedSimpleAdd"));
		return suite;
	}

}
