
/**
 * This Stack implementation uses a linked list of Cells.<p>
 * <i>NB:</i> LinkStack is <i>not</i> thread safe!
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 1.0 1998-11-25
 */
public class LinkStack implements StackInterface {
	private Cell top_;
	private int size_;
	
	public LinkStack() {
		// Establishes the invariant.
		top_ = null;
		size_ = 0;
	}
	
	/**
	 * This inner class is just a glorified data structure
	 * to hold items and links to other cells.
	 */
	public class Cell {
		public Object item;
		public Cell next;
		public Cell(Object item, Cell next) {
			this.item = item;
			this.next = next;
		}
	}
	
	public boolean isEmpty() { return this.size() == 0; }
	public int size() { return size_; }
	
	/**
	 * Links a new Cell to the head of the list
	 * pointed to by top_.
	 */
	public void push(Object item) throws AssertionException {
		top_ = new Cell(item, top_);
		size_++;
		assert(!this.isEmpty());
		assert(this.top() == item);
		assert(invariant());
	}
	
	public Object top() throws AssertionException {
		assert(!this.isEmpty());
		return top_.item;
	}
	
	/**
	 * Unlinks the head Cell and sets top_ to
	 * point to the tail of the list.
	 */
	public void pop() throws AssertionException {
		assert(!this.isEmpty());
		top_ = top_.next;
		size_--;
		assert(invariant());
	}
	
	/**
	 * The invariant is established by the constructor, and
	 * must hold at the start <I>and</i> end of each method.
	 * (To keep things simple, we only check at the end of
	 * methods that modify the state.)
	 */
	private boolean invariant() {
		return (size_ >= 0) &&
			((size_ == 0 && this.top_ == null)
			|| (size_ > 0 && this.top_ != null));
	}
	
	/**
	 * There are various kind of assertions.
	 * In this example we focus on <i>class invariants</i>,
	 * which must hold before and after each public method,
	 * <i>preconditions</i> for methods (AKA "requires" clauses)
	 * and <i>postconditions</i> (AKA "ensures" clauses).
	 * <p>
	 * Also useful are <i>loop invariants</i> (none here),
	 * and any kind of assertion of any non-trivial property that
	 * you expect to hold at some point in your program.
	 * (As you might find in unit tests.)
	 */
	private void assert(boolean assertion)
		throws AssertionException
	{
		if (!assertion) {
			throw new AssertionException("Assertion failed in LinkStack");
		}
	}
}