/**
 * Class to enforce rules of TicTacToe.
 * @author Oscar.Nierstrasz@acm.org
 * @version 1.1 1999-01-19
 */
public class TicTacToe {

	private char[][] gameState_;

	/**
	 * The state of the game is represented as 3x3
	 * array of chars marked ' ', "x", or 'O'.
	 * We index the state using chess notation,
	 * i.e., column is 'a' through 'c' and row is
	 * '1' through '3'.
	 */
	public TicTacToe() {
		gameState_ = new char[3][3];
		for (char col='a'; col <='c'; col++)
			for (char row='1'; row<='3'; row++)
				this.set(col,row,' ');
	}
	
	/**
	 * set() and get() translate between chess coordinates
	 * and array indices.
	 */
	private void set(char col, char row, char mark) {
		assert(inRange(col, row));
		gameState_[col-'a'][row-'1'] = mark;
	}

	private char get(char col, char row) {
		assert(inRange(col, row));
		return gameState_[col-'a'][row-'1'];
	}

	/**
	 * For now, there is no implementation of the
	 * game behaviour, so the game is immediately over.
	 */
	public boolean notOver() {
		return false;
	}
	
	/**
	 * A plain ascii representation of the game,
	 * mainly for debugging purposes.
	 */
	public String toString() {
		StringBuffer rep = new StringBuffer();
	
		for (char row='3'; row>='1'; row--) {
			rep.append(row);
			rep.append("  ");
			for (char col='a'; col <='c'; col++) {
				rep.append(this.get(col,row));
				if (col<'c') {
					rep.append(" | ");
				}
			}
			rep.append('\n');
			if (row>'1') {
				rep.append("  ---+---+---\n");
			}
		}
		rep.append("   a   b   c\n");
		return(rep.toString());
	}
	
	private boolean inRange(char col, char row) {
		return (('a'<=col) && (col<='c')
			&& ('1'<=row) && (row<='3'));
	}

	/**
	 * For now, rather than pass assertion exceptions
	 * on to clients, we catch them here and exit().
	 * Should be protected? public?
	 */
	private void assert(boolean assertion) {
		if (!assertion) {
			try {
				throw new AssertionException("assertion failed in "
					+ this.getClass().getName());
			} catch (AssertionException err) {
				err.printStackTrace();
				System.exit(1);
			}
		}
	}

	/**
	 * For now, just test some obvious things.
	 * Do we need to test toString()?  How?
	 */
	public void test() {
		System.err.println("Started TicTacToe tests");
		assert(this.get('a','1') == ' ');
		assert(this.get('c','3') == ' ');
		this.set('c','3','X');
		assert(this.get('c','3') == 'X');
		this.set('c','3',' ');
		assert(this.get('c','3') == ' ');
		assert(!this.inRange('d','4'));
		System.err.println("Passed TicTacToe tests");
	}
}
