import java.io.*;

/**
 * Manage interaction with user.
 * @author Oscar.Nierstrasz@acm.org
 * @version 1.5 1999-02-07
 */
public class Player {
	private final char mark_;
	private final BufferedReader in_;

	/**
	 * Constructor to specify an alternative source of moves
	 * (e.g., a test case StringReader).
	 */
	protected Player(char mark, BufferedReader in) {
		mark_ = mark;
		in_ = in;
	}
	
	/**
	 * The normal contructor to use:
	 */
	public Player(char mark) {
		this(mark, new BufferedReader(new InputStreamReader(System.in)));
	}

	/**
	 * Special constructor to make a Player that plays a fixed
	 * set of moves from a String.  Used to define test cases.
	 */
	public Player(char mark, String moves) {
		this(mark, new BufferedReader(new StringReader(moves)));
	}
	
	/**
	 * Special constructor for the Player representing nobody.
	 * (Used by TicTacToe as a placeholder when there is no
	 * winner yet.)
	 */
	public Player() {
		this(' ');
	}

	public char mark() { return mark_; }
	
	public String name() {
		if (this.isNobody())
			return "nobody";
		else
			return new Character(this.mark()).toString();
	}
	
	/**
	 * By convention, a Player without a mark is nobody!
	 */
	public boolean isNobody() {
		return this.mark() == ' ';
	}

	public void move(BoardGame game)
		throws IOException
	{
		String line = in_.readLine();
		if (line == null)
			throw new IOException("end of input");
		try {
			game.move(line, this.mark());
		} catch (AssertionException err) {
			System.err.println("Invalid move ignored (" + line + ")");
			// err.printStackTrace();
		}
	}
}
