import java.io.*;
/**
 * Helper class to find winning scores for BoardGame.
 * @author Oscar.Nierstrasz@acm.org
 * @version 1.6 1999-02-04
 */
public class Runner {

	BoardGame game_;
	int homeCol_, homeRow_;			// Home col and row
	int col_=0, row_=0;				// Current col & row

	/**
	 * This class does not have any interesting
	 * contracts.  We do not bother to define
	 * a separate test() method, since it will
	 * be thoroughly tested by the unit tests
	 * for the BoardGame implementations.
	 */
	public Runner(BoardGame game, int col, int row)
	{
		game_ = game;
		homeCol_ = col;
		homeRow_ = row;
	}

	/**
	 * A Runner runs in some direction (dcol,drow)
	 * as far as it can, as long as the pieces on the
	 * Board are the same as the home square.
	 * Then it runs in the opposite direction, and
	 * returns the total score.
	 */
	public int run(int dcol, int drow)
		throws AssertionException
	{
		int score = 1;
		this.goHome();
		score += this.forwardRun(dcol, drow);
		this.goHome();
		score += this.reverseRun(dcol, drow);
		return score;
	}
	
	private void goHome() {
		col_= homeCol_;
		row_ = homeRow_;
	}
	
	/**
	 * Recursively move forward as long as we are
	 * in a run.  Return the length of the run.
	 */
	private int forwardRun(int dcol, int drow)
		throws AssertionException
	{
		this.move(dcol, drow);
		if (this.samePlayer())
			return 1 + this.forwardRun(dcol, drow);
		else
			return 0;
	}
	
	/**
	 * Reverse direction of running.
	 */
	private int reverseRun(int dcol, int drow)
		throws AssertionException
	{
		return this.forwardRun(-dcol, -drow);
	}
	
	private void move(int dcol, int drow) {
		col_ = col_ + dcol;
		row_ = row_ + drow;
	}

	/**
	 * The player at the current location is the
	 * same as that of our home square.
	 */
	private boolean samePlayer()
		throws AssertionException
	{
		if (game_.inRange(col_,row_)) {
			char home = game_.get(homeCol_, homeRow_);
			char here = game_.get(col_, row_);
			return home == here;
		} else {
			// Oops -- we walked off the edge of the board!
			return false;
		}
	}
}
