package client;

import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;

import tictactoe.GameFactory;
import tictactoe.RemoteGame;
import tictactoe.Move;

/**
 * This class implements the view on a TicTacToe game.
 * The Applet contains a grid of Place objects that
 * intercept mouse clicks and forward them to the game's
 * Players.  The Applet is also an Observer for the game.
 * When the game notifies the Applet of state change, it
 * updates the corresponding Place.  Below the grid there
 * is a Label component to display messages.
 *
 * @author Oscar.Nierstrasz@acm.org
 * @version 2.1 1999-03-31
 */
public class GameApplet extends Applet
{	
	/**
	 * Return author and copyright information
	 */
	public String getAppletInfo() {
		return "Author Oscar.Nierstrasz@acm.org (c) 1999";
	}

	/**
	 * Instantiates the BoardGame, initializes the view components,
	 * and connects the view to the model.
	 */
	public void init() {
		setLayout(new FlowLayout());

		this.add(makeControls());

		// Initialize images for Places (static class method)
		Place.setImages(
			getImage(getCodeBase(), "images/cross.gif"),
			getImage(getCodeBase(), "images/not.gif")
		);
	}

	private Component makeControls() {
		Panel controls = new Panel();
		controls.setLayout(new FlowLayout());

		Button again = new Button("New game");
		again.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// NB: anonymous inner class has access to
				// methods of enclosing class (GameApplet)
				showStatus("starting new game ...");
				GameView view = new GameView();
				view.show();
			}
		});
		controls.add(again);

		return controls;
	}
}
